/*
 * Javascript which handles tree display and navigation
 */


var trees             = {};       // a hash table which represents the trees

var delimeter = "\u0007";           // a bell delimeter
var currentNode = null;             // currently hovered node
var popupNode = null;               // the node on which a popup is displayed
var popup2show = null;              // the popup to show
var timerId = null;                 // the timer for the popups
var timeOutvalue = 500;             // the timeout value in milliseconds
var popupX = 0;                     // the global x co-ordinate to display the popup
var popupY = 0;                     // the global y co-ordinate to display the popup

var mouseOverColor = "#E0E0E0";     // the mouseover color
var linkBgColor = "#FFFFFF";        // background color of the popup
var popupTextColor = "#000000";     // the text of the popup
var popupRolloverColor = '#347EDA'; // roll over color of the popup
var selectedNodeColor = '#C0DBF7';  // the color of the selected node

var textNode = "node";              // i18n Text for node
var textCollapse = "Collapse";      // i18n Text
var textCollapsed = "Collapsed";    // i18n Text
var textExpand = "Expand";          // i18n Text
var textExpanded = "Expanded";      // i18n Text
var textLevel = "Level";            // i18n Text
var textOf = "of";                  // i18n Text
var textSelected = "Selected";      // i18n Text

function Tree(name, rootNode, baseDir, isLocal, selectedNode)
{
    this.name = name;
    this.baseDir = baseDir;
    this.isLocal = isLocal;
    this.rootNode = rootNode;
    this.icons = new Array(6);              // the tree icons
    this.openNodes = {};                    // used to store all open nodes (if isLocal is true).
    this.highlightNodePath = selectedNode;  // each tree can optionally have one highlighted node
    this.popups = {};                       // all the defined popups for the tree.
    this.recursedNodes = new Array();       // temporary variable used in tree generation   

    this.preLoadIcons = 
       function preLoadIcons()
       {
	      this.icons[0] = new Image();
          this.icons[0].src = baseDir + "plus.gif";
          this.icons[1] = new Image();
          this.icons[1].src = baseDir + "plusbottom.gif";
          this.icons[2] = new Image();
          this.icons[2].src = baseDir + "minus.gif";
          this.icons[3] = new Image();
          this.icons[3].src = baseDir + "minusbottom.gif";
          this.icons[4] = new Image();
          this.icons[4].src = baseDir + "folder.gif";
          this.icons[5] = new Image();
          this.icons[5].src = baseDir + "folderopen.gif";
       }

    // recursive function which finds a given node in the tree 
    this.getNode = 
       function getNode(parent, nodePath)
       {
           if(parent.path == nodePath) {
              return parent;
           }
              
           for(var i=0; i<parent.children.length; i++)
           {
              var n = getNode(parent.children[i], nodePath);
              if(n != null)
                 return n;
           }
           
           return null;
       }

    // get a particular popup for the tree
    this.getPopup = 
       function getPopup(popupName)
       {
          // if it has already been created, return it.
          var popup = document.getElementById(this.name+"_popups_"+popupName);
          if(popup != null)
             return popup;         

          // let us see if the popup is in the popups hashtable
          popup = this.popups[popupName]
          if(popup == null)
             return null;   // does not exist, so just return a null.

          var popupPlaceholder = document.getElementById(this.name+'popupPlaceholder');

          // create the table for the popup
          var table = document.createElement('table');
          table.setAttribute('id', this.name+"_popups_"+popupName);
          table.setAttribute('cellpadding', '1');
          table.setAttribute('cellspacing', '0');
          table.setAttribute('class', 'popup');
          table.setAttribute('onMouseOver', 'popupCancelhide("'+this.name+'");');
          table.setAttribute('onMouseOut', 'popupHide("'+this.name+'");');

          var tbody = document.createElement('tbody');
	
          for(var i=0; i<popup.length; i++)
          {
             treePopup = popup[i];
             row = document.createElement('tr');
             column = document.createElement('td');
             columnClass = (i == popup.length-1 ? 'popup_td_bottom' : 'popup_td');

             column.setAttribute('nowrap', true);
             column.setAttribute('class', columnClass);
             column.setAttribute('onMouseOver', 'popupRollOver(this);');
             column.setAttribute('onMouseOut', 'popupRollOut(this);');
             column.setAttribute('onClick', treePopup.url);
			 column.setAttribute('valign', 'top');

             // the popup has an image associated with it. let us show it.
			 var elem = document.createTextNode(' ');
             if(treePopup.imageSrc != null && treePopup.imageSrc != undefined)
			 {
                 elem = document.createElement('img');
				 elem.setAttribute('src', treePopup.imageSrc);
			 }		 
			 
             if(document.all)
                column.setAttribute('style', 'cursor:hand;');
             else
                column.setAttribute('style', 'cursor:pointer;');

             spanE = document.createElement('span');
			 spanE.setAttribute('class', 'popup_text');
             text = document.createTextNode(' ' + treePopup.text);
			 spanE.appendChild(text)

             column.appendChild(elem);
             column.appendChild(spanE);

             row.appendChild(column);
             tbody.appendChild(row);
          }
          table.appendChild(tbody);

          if(document.all) // bug in IE requires this code.....
             popupPlaceholder.innerHTML += table.outerHTML;
          else
             popupPlaceholder.appendChild(table);

          return table;
       }

    /**
     * Render the given tree
     *
     */
    this.renderTree = 
       function renderTree()
       {    
           this.preLoadIcons();

           var node = this.rootNode;

           if(node == null)
              return; // nothing can be displayed in the tree.

           // if setLocal=true, load all the openNodes from cookie
           if(this.isLocal)
              retrieveOpenNodesFromCookie(name);

           writeNode(node, node.iconOpen, (tree.highlightNodePath == node.name ? true : false), node.name);
           document.write('<br />');

           this.addNode(node);

           if(tree.highlightNodePath != null)  
              setTimeout("scrollToElement('"+name+"','"+ tree.highlightNodePath +"')", 300);

       }
    
    // recursive function to keep adding nodes to tree
    this.addNode = 
       function addNode(parentNode)
       { 
          for (var n=0; n<parentNode.children.length; n++)
          {
             var node = parentNode.children[n];  // find the node

             var id = node.treeName + delimeter + node.ePath;
             var ls = (n==parentNode.children.length-1 ? true : false);
             var ino = node.isExpanded;
             var ish = (this.highlightNodePath == node.path);
			
             // if the node is not open and setLocal=true, then let us see if that node exists in the openNodes
	         // hashTable.
             if(!ino && isLocal)
             {
                if(tree.openNodes[node.path] != null)
	            ino = true;
             }
			
             // Write out line & empty icons
             for (g=0; g<this.recursedNodes.length; g++)
             {
               if (this.recursedNodes[g] == 1)
                 document.write('<img alt="" border=0 src="' + this.baseDir + 'line.gif" align="absbottom" />');
               else
                 document.write('<img alt="" border=0 src="' + this.baseDir + 'empty.gif" align="absbottom" />');
             }

             // put in array line & empty icons

             if (ls)
               this.recursedNodes.push(0);
             else
               this.recursedNodes.push(1);

             // Write out join icons
             if (node.hasChildren)
             {
                 // generate links based on local variable or not
                 var callFunction = '';
                 if(this.isLocal == true)
                 {
                   callFunction ="ocLocal('"+node.treeName+"','";
                 } else {
                   callFunction ="oc('"+node.treeName+"','";
                 }

                 var titleText = " " + textNode + " " + node.name;
                 var temp = '';

                 if (ls)
                   temp = 'bottom.gif';
                 else
                   temp = '.gif';
                 
                 document.write('<a href="javascript:' + callFunction + node.ePath + '\',1);"><img border=0 id="join' + id + '" src="');
 
                 if (ino) {
                   document.write(this.baseDir + 'minus');
                   titleText = textCollapse + titleText;
                 } else {
                   document.write(this.baseDir + 'plus');
                   titleText = textExpand + titleText;
                 }

                 document.write(temp+'" align="absbottom" title="' + titleText + '" alt="' + titleText + '" /></a>');

             } else {
                 if (ls)
                   document.write('<img alt="" border=0 src="' + this.baseDir + 'join.gif" align="absbottom" />');
                 else
                   document.write('<img alt="" border=0 src="' + this.baseDir + 'joinbottom.gif" align="absbottom" />');
             }

             // Write out folder & page icons
             var imgsrc = "";
             if (node.hasChildren)
             {
               if(ino)
                  imgsrc = node.iconOpen;
               else
                  imgsrc = node.iconClosed;
             } else {
               imgsrc = node.iconClosed;
             }

             // write out the node alternate information for accessibility purpose
             var altInfo = textLevel + " " + this.recursedNodes.length + ", ";
             if(node.hasChildren)
             {
                if(ino)
                  altInfo += textExpanded + ", ";
                else
                  altInfo += textCollapsed + ", ";
             }
             altInfo += node.name + ", " + (n+1) + " " + textOf + " " + parentNode.children.length;
		
             if(ish)
               altInfo += ", " + textSelected;

             writeNode(node, imgsrc, ish, altInfo);
             document.write("<br />");

             // If node has children go deeper and write divs
             if (node.hasChildren)
             {
                document.write('<div id="div' + id + '"');
                if (!ino) document.write(' style="display: none;"');
                document.write('>');
                if(ino || isLocal)
                   this.addNode(node);

                document.write('</div>');
             }

             // remove last line or empty icon
             this.recursedNodes.pop();
	      } // for
	   } // end function

    // add the tree to the hashtable    
    trees[name] = this;
}


function TreeNode(path, name, formName, iconOpen, iconClosed, popupText, popupId, isExpanded, hasChildren, treeName, preSubmitAction)
{
    this.path = path;                    this.ePath = escapeQuotes(path);
    this.name = name;                 
    this.action = 'javascript:selectNodeAction(\''+treeName + '\',\''+ this.ePath+ '\',\''+ formName +'\')";';
    this.formName = formName;
    this.iconOpen = iconOpen;
    this.iconClosed = iconClosed;
    this.popupText = popupText;
    this.popupId = popupId;
    this.parent = null;
    this.isExpanded = isExpanded;
    this.hasChildren = hasChildren;
    this.treeName = treeName;

	if(preSubmitAction != undefined && preSubmitAction != null)
	{
	   this.preSubmitScript = preSubmitAction;
	}

    this.children = new Array();   
    
    this.addChild = 
       function addChild(path, name, formName, iconOpen, iconClosed, popupText, popupId, isExpanded, hasChildren, treeName, preSubmitAction)
       {
          var child = new TreeNode(path, name, formName, iconOpen, iconClosed, popupText, popupId, isExpanded, hasChildren, treeName, preSubmitAction);
          child.parent = this;
          this.children.push(child);
          
          return child;
       }
        
}

/**
 * The TreePopup Object which holds each popup item. These items are then copied into the popup array
 *
 * @param text The text to be displayed on the popup
 * @param url The url to be executed for the popup
 * @param imageSrc The imageSrc for the popup
 */
function TreePopup(text, url, imageSrc)
{
   this.text = text;
   this.url = url;
   this.imageSrc = imageSrc;
}



/**
 * Write an invidividual node out using document.write. The reason it is separated is so that
 * if someone wants to use treenodes other than a Tree (ie in a list) and still get the same
 * functionality, they can just call this method
 *
 * @param node      The TreeNode object to be written
 * @param icon      Optional - The icon source to be displayed along with the node. If icon is null, no icon
 *                  will be displayed next to the node.
 * @param highlight Optional parameter, set to true if we want the node to be highlighted
 * @param accessibility  The alternate text to be used for accessibility. If not specified, a blank will be used
 */
function writeNode(node, icon, highlighted, accessibility )
{
   var class_name = 'tree_link';
   var id = node.treeName + delimeter + node.ePath;

   // highlight the open node
   if(highlighted)
      document.write('<span class="selectedNode">');

   var mouseOverString   = ' onMouseOver="mouseOverCall(\''+ node.treeName + '\',\'' + node.ePath + '\',\'' +  node.popupId + '\');"';
   var mouseOutString    = ' onMouseOut="mouseOutCall(\''+ node.treeName + '\',\'' + node.ePath + '\');"  ';
   var contextMenuString = ' oncontextmenu="return startDisplayPopup(\''+ node.treeName + '\',\'' + node.ePath +'\',\'' +  node.popupId + '\', event);"  ';
   var titleString       = (node.popupText != null ? ' title="' + escapeQuotes(node.popupText) + '"' : '');
      
   // used in the href
   var altTextString     = ' title="' + (accessibility != null ? escapeQuotes(accessibility) : '') + '"';
   
   if(icon != null)
   {
       document.write('<a ' + altTextString + ' href="' + node.action + '">');
	   document.write('<img id="icon' + id + '" src="' + icon + '" align="absbottom"  alt="" border="0" ' + mouseOverString + mouseOutString + contextMenuString + titleString + '/>');
       document.write('</a>');
   }

   // Start link - include an id for highlighting
   var writeString = '<a class="' + class_name  + '" id="link' + id + '" href="' + node.action + '"  onKeyPress="checkKeyStroke(\''+ node.treeName + '\',event);" ' + mouseOverString +  mouseOutString  + contextMenuString + '/>';
   document.write(writeString);

   if(node._prefix != undefined)
      document.write(node._prefix);

   document.write(htmlChars(node.name));

   if(node._suffix != undefined)
      document.write(node._suffix);

   // End link
   document.write('</a>');

   if(highlighted)
      document.write('</span>');

   //This span is an empty placeholder for users to do an 'innerHTML()' call on
   // to add things directly after the node (ie selection images)
   document.write('<span id="placeholder' + id + '"></span>');
}

/** 
 * Executes a node form action after a user clicks on a tree node.
 */
function selectNodeAction(treeName, nodePath, formName)
{
    if(formName == null || formName == "null")
        return;

    var tree = trees[treeName];
    var node = tree.getNode(tree.rootNode, nodePath);
    
	// if the tree has a presubmit action, we want to evaluate it also
	if(node.preSubmitScript != null && node.preSubmitScript != undefined)
	   eval(node.preSubmitScript);
	
    // we want to add some extra parameters to the form
    addExtraParameters(formName, treeName + "highlightNodePath", nodePath);   // we want to select that node
    addExtraParameters(formName, treeName + "highlightNodeData", node.data);  // also get it's data field.        
    addExtraParameters(formName, treeName + "highlightNodeName", node.name);  // also get it's name.
	
    // now we want to submit the form
    var action = "javascript:document."+formName+".submit()";
    eval(action);
}


/**
 * javascript called when the mouse is over a node.
 * @param treeName The name of the tree
 * @param nodePath The nodePath
 * @popupShowId Optional parameter. If specified, it will show the
 *              popup with the specified id, else the default will
 *              be shown. Default is 'popup'
 */
function mouseOverCall(treeName, nodePath, popupShowId)
{
    var tree = trees[treeName];
    var node = tree.getNode(tree.rootNode, nodePath);  
	
    // hide any earlier popup's
    displayPopup(treeName, 'hidden');

    if(popupShowId != null)
    {
       if(popupShowId != popup2show)
       {
          currentNode = node;
       }
       popup2show = popupShowId;
    }

	currentNode = node;

	// color the link to give the appearance of being in focus.
    var id = node.treeName + delimeter + node.ePath;
	var element = getLinkElement(id);
   
    if(tree.highlightNodePath != nodePath)
  	    element.style.backgroundColor = mouseOverColor;
  	else
        element.style.backgroundColor = selectedNodeColor;
}

// javascript called when the mouse moves outside a node
function mouseOutCall(treeName, nodePath)
{
   popupHide(treeName);

   var escapedAgainId = escapeQuotes(nodePath);
   var id = treeName + delimeter + escapedAgainId;
   var tree = trees[treeName];

   // This will color the link back to it's original background color, making it look out of focus
   var element = getLinkElement(id);
   if(tree.highlightNodePath != nodePath)
      element.style.backgroundColor = linkBgColor;
   else
      element.style.backgroundColor = selectedNodeColor;
}

// function which is going to display the popup
function startDisplayPopup(treeName, nodePath, popupShowId, event)
{  
   // if popupShowId is null, we want to show the default popup menu, so return true.
   if(popupShowId == null || popupShowId == 'null')
      return true;

   if(document.all)
   {
      popupX = event.x + document.body.scrollLeft;
      popupY = event.y + document.body.scrollTop;
   } else {
      // try the standard
      popupX = event.pageX;
      popupY = event.pageY;
   }

   clearTimer();

   // call the mouseOverCall - just in case
   mouseOverCall(treeName, nodePath, popupShowId);

   // let us set this guy before we display the popup, so that it is available to the popup action url's.
   popupNode = currentNode;
   
   // no need to wait, just display the popup
   displayPopup(treeName, 'visible');

   var escapedAgainId = escapeQuotes(nodePath);
   var id = treeName + delimeter + escapedAgainId;
   var element = getLinkElement(id);

   var tree = trees[treeName];
   if(tree.highlightNodePath != nodePath)
   {
       element.style.backgroundColor = mouseOverColor ;
   }

   // since we do not want to display the context menu.
   return false;
}

/**
 * function which starts the timer that is going to display or hide the popup.
 * @param treeName the name of the tree
 * @param flag - can be 'visible' or 'hidden';
 */
function displayPopup(treeName, flag)
{
   var popupObject = null;

   // even though multiple trees can be displayed on a page, only one popup menu is visible.
   if(popup2show == null)
   {
      return;
   }

   var tree = trees[treeName];

   popupObject = tree.getPopup(popup2show);
   if(popupObject == null)
       return;

   popupObject.style.left = popupX;
   popupObject.style.top  = popupY;
   popupObject.style.visibility = flag;
}

// function called from the popup to hide it
function popupHide(treeName)
{   
    if(currentNode != null)
    {
       var id = treeName + delimeter + currentNode.ePath;
       var element = getLinkElement(id);
       element.style.backgroundColor = linkBgColor;
    }
    
    clearTimer();
    var fStr = "displayPopup('"+treeName+"','hidden')";
    timerId = setTimeout(fStr, timeOutvalue);
}

// function called from the popup to show it, basically kills the timer meant to hide the popup.
function popupCancelhide(treeName)
{  
    if(currentNode != null)
    {
       var id = treeName + delimeter + currentNode.ePath;
       var element = getLinkElement(id);
       element.style.backgroundColor = mouseOverColor;
    }
	clearTimer();
}

// rollover function for the popup td
function popupRollOver(td)
{
   td.style.backgroundColor=popupRolloverColor;
   td.style.color=linkBgColor;
}

function popupRollOut(td)
{
   td.style.backgroundColor = mouseOverColor;
   td.style.color=popupTextColor;
}

// clear any prior timer if it exists
function clearTimer()
{
	if(timerId != null)
	{
    	   clearTimeout(timerId);
    	   timerId = null;
    	}
}

/** Scroll to the selected element in the tree */
function scrollToElement(treeName, nodePath)
{
  // after creating the tree, we need to go give the specified selected node
  var sElement = getLinkElement(treeName + delimeter + escapeQuotes(nodePath));
  if(sElement != null)
    sElement.focus();

}

// Opens or closes a node by submitting a form
function oc(treeName, nodePath)
{
    var id = treeName + delimeter + nodePath;
	var tree = trees[treeName];
	var node = tree.getNode(tree.rootNode,nodePath);
	var theDiv = document.getElementById('div' + id);
	var uId = null;

	if(theDiv == null)
	{ 
	    uId = escapeQuotes(nodePath);
	    theDiv = document.getElementById('div' + uId);
    }

	if(theDiv == null)      // could be due to a double quote
	{
	    uId = replaceDoubleQuotes(uId);
		theDiv = document.getElementById('div' + uId);
	}

    // the image div
    var theJoin = document.getElementById('join' + id);
	
	var formName = "document."+treeName+"ManagementForm.";
	var formSubmit = formName + "submit()";

	if (theJoin.src.indexOf('plus.gif') != -1 || theJoin.src.indexOf('plusbottom.gif') != -1) // node is to be opened
	{
	    var x = formName + treeName + "expandNode.value='"+nodePath+"'";
	    eval(x);
	    eval(formSubmit);
	} else  // node is closed
	{
	    
	    // if a node below the current node is selected, we have to automatically select this node after collapsing it
	    if(tree.highlightNodePath != null && tree.highlightNodePath.indexOf(nodePath) == 0)
	    {
	        // we have to click on this node and submit its form
            addExtraParameters(node.formName, treeName + "collapseNode",nodePath);        // collapse the node
            addExtraParameters(node.formName, treeName + "highlightNodePath",nodePath);   // we want to select that node
            addExtraParameters(node.formName, treeName + "highlightNodeData",node.data);  // also get it's data field.        
            eval(node.action);
	    } else
	    {
            // just collapse the node without submitting it
	        var x = formName + treeName + "collapseNode.value='"+nodePath+"'";
            eval(x);
            eval(formSubmit);
	    }
	}
}

// opens or closes a node locally
function ocLocal(treeName, nodePath, bottom)
{
    var id = treeName + delimeter + nodePath;
	var theDiv  = document.getElementById('div' + id);
	var theJoin = document.getElementById('join' + id);
	var theIcon = document.getElementById('icon' + id);
	var uId = null;
	var tree = trees[treeName];

	if(theDiv == null)
	{ 
	    uId = escapeQuotes(id);
	    theDiv = document.getElementById('div' + uId);
		if(theJoin == null) theJoin	= document.getElementById('join' + uId);
		if(theIcon == null) theIcon = document.getElementById('icon' + uId);
    }
	
	if(theDiv == null)      // could be due to a double quote
	{
	    uId = replaceDoubleQuotes(uId);
		theDiv = document.getElementById('div' + uId);
        if(theJoin == null) theJoin	= document.getElementById('join' + uId);
		if(theIcon == null) theIcon = document.getElementById('icon' + uId);
	}

    // if theJoin.src contains plus.gif, it means that the node is asked to be expanded
	// if theJoin.src contains minus.gif, it means that the node is asked to be collapsed
    if(theJoin.src.indexOf("plus.gif") != -1 || theJoin.src.indexOf('plusbottom.gif') != -1)
	   tree.openNodes[nodePath] = "true";
	else
	   tree.openNodes[nodePath] = null;
	     
    storeOpenNodesInCookie(treeName);

	if (theJoin.src.indexOf('plus.gif') != -1 || theJoin.src.indexOf('plusbottom.gif') != -1) // node is to be opened
	{
		if (bottom==1) theJoin.src = tree.icons[3].src;
		else theJoin.src = tree.icons[2].src;
		theIcon.src = tree.getNode(tree.rootNode,nodePath).iconOpen;
		theDiv.style.display = '';
	} else
	{
		if (bottom==1) theJoin.src = tree.icons[1].src;
		else theJoin.src = tree.icons[0].src;
		theIcon.src = tree.getNode(tree.rootNode,nodePath).iconClosed;
		theDiv.style.display = 'none';
	}
}


// store the openNodes in the cookie for the tree.
// @param name The name of the tree
function storeOpenNodesInCookie(name)
{
   var tree = trees[name];
   var cookieName = name + "openNodes";
   var cookieValue = "";
    
   for(var n in tree.openNodes)
   {
      if(tree.openNodes[n] != null)
         cookieValue += n + delimeter;
   }

   document.cookie = cookieName + "=" + escape(cookieValue);
}

// this method is used to add extra attributes to a specified form
function addExtraParameters(formName, param, value)
{  
   if(formName == null || formName == 'null' || param == null || value == null || param == "" || value == "" || value == 'null') 
   {
      return;
   }
   var x = "var treeForm = document."+formName;
   eval(x);

   var hiddenNode = document.createElement('input');
   hiddenNode.setAttribute('type','hidden');
   hiddenNode.setAttribute('name',param);
   if(value.value != undefined)
       hiddenNode.setAttribute('value',value.value);        // so that users can put in text field as values
   else
	   hiddenNode.setAttribute('value',value);

   treeForm.appendChild(hiddenNode);
}

// add a node form element to the popupPlaceHolder element
// used by the Node to generate a new form element in case of a url
function addFormElement(treeName, formName, url)
{
    var formString = "<form name='"+formName+"' method='post' action='"+url+"'></form>";
    var id = document.getElementById(treeName + "popupPlaceholder");
    id.innerHTML += formString;
}


// retrieve openNodes from cookie
// @param name The name of the tree
function retrieveOpenNodesFromCookie(name)
{
   var tree = trees[name];
   var arr = document.cookie.split(";");

   if(arr == null || arr.length == 0)
      return;
  
   for(i=0; i<arr.length; i++)
   {
      if(arr[i].indexOf(name+"openNodes") != -1)
      {
          var j = arr[i].indexOf("=");
          if(j != -1)
          {
             var cookieValue = unescape(arr[i].substring(j+1));
             
             // now let us create an split this thing...
             var valueArray = cookieValue.split(delimeter);

             if(valueArray == null || valueArray.length == 0)
                return;
             
             for(k=0; k<valueArray.length; k++)
             {
                 if(valueArray[k].length > 0) {
                    tree.openNodes[valueArray[k]] = "true";
                 }
             } // end for
          } // endif j != -1
      } // endif arr[i].indexOf(openNodes)
   } // endfor
}


/**
 * Utility functions for the script
 */

// get the link represented by the id
function getLinkElement(id)
{
   var lid = "link" + id;
   var element = document.getElementById(lid);

   if(element != null)
       return element;

   // there must be a double quote in there,
   var decode = lid.replace(/&quot;/g, '"');
   element = document.getElementById(decode);
   return element;

}

// destroys the popup if escape pressed
function checkKeyStroke(treeName, event)
{
  if(event.keyCode == 27)
     displayPopup(treeName, 'hidden');
}

// escape the single quotes out of the id....
function escapeQuotes(id)
{
    var n = id.replace(/'/g, '\\\u0027');
	n = escapeDoubleQuotes(n);
	return n;
}

function escapeDoubleQuotes(id)
{
    return id.replace(/"/g, '&quot;');
}

function replaceDoubleQuotes(id)
{
    return id.replace(/&quot;/g, '"');
}

function replaceQuotes(id)
{  
    var n = id.replace(/\\u0027/g, "'");
	n = n.replace(/\\u0022/g, '"');
    return n;
}

function reverseQuotesToUnicode(id)
{
    var n = id.replace(/'/g, "\\u0027");
	n = n.replace(/"/g, '\\u0022');
    return n;
}

function htmlChars(source)
{
   if(source == null)
       return source;
            
   var target = source;

   target = target.replace(/&/g, "&amp;");
   target = target.replace(/</g, "&lt;");
   target = target.replace(/>/g, "&gt;");

   return target;
}

 
// simple trim function
function strtrim()
{
    //Match spaces at beginning and end of text and replace
    //with null strings
     return this.replace(/^\s+/,'').replace(/\s+$/,'');
}
String.prototype.trim = strtrim;
